@@ -8,7 +8,7 @@ use chrono::{NaiveTime, Utc};
 use currency::Currency;
 use dateparser::parse_with;
 use eudex::Hash;
-use indicatif::ProgressBar;
+use indicatif::{ProgressBar, ProgressDrawTarget};
 use once_cell::sync::OnceCell;
 use regex::Regex;
 use reverse_geocoder::{Locations, ReverseGeocoder};
@@ -18,6 +18,8 @@ use strsim::{
     sorensen_dice,
 };
 use titlecase::titlecase;
+use vader_sentiment::SentimentIntensityAnalyzer;
+use whatlang::detect;
 
 static USAGE: &str = "
 Apply a series of transformation functions to a given CSV column. This can be used to
@@ -42,11 +44,11 @@ Currently supported operations:
   * trim: Trim (drop whitespace left & right of the string)
   * ltrim: Left trim whitespace
   * rtrim: Right trim whitespace
-  * mtrim: Trims --comparand matches left & right of the string
-  * mltrim: Left trim --comparand matches
-  * mrtrim: Right trim --comparand matches
+  * mtrim: Trims --comparand matches left & right of the string (Rust trim_matches)
+  * mltrim: Left trim --comparand matches (Rust trim_start_matches)
+  * mrtrim: Right trim --comparand matches (Rust trim_end_matches)
   * replace: Replace all matches of a pattern (using --comparand)
-      with a string (using --replacement).
+      with a string (using --replacement) (Rust replace)
   * regex_replace: Replace all regex matches in --comparand w/ --replacement.
   * titlecase - capitalizes English text using Daring Fireball titlecase style
       https://daringfireball.net/2008/05/title_case 
@@ -66,6 +68,9 @@ Currently supported operations:
   * simhm: Hamming distance to --comparand. Num of positions characters differ.
   * simod: OSA Distance to --comparand.
   * eudex: Multi-lingual sounds like --comparand (boolean)
+  * sentiment: Normalized VADER sentiment score (English only - between -1.0 to 1.0).
+  * whatlang: Language Detection. For 87 supported languages, see
+       https://github.com/greyblake/whatlang-rs/blob/master/SUPPORTED_LANGUAGES.md
 
 Examples:
 Trim, then transform to uppercase the surname field.
@@ -231,6 +236,8 @@ static OPERATIONS: &[&str] = &[
     "simhm",
     "simod",
     "eudex",
+    "sentiment",
+    "whatlang",
 ];
 
 #[derive(Deserialize, Debug)]
@@ -259,6 +266,7 @@ static LOCS: OnceCell<Locations> = OnceCell::new();
 static GEOCODER: OnceCell<ReverseGeocoder> = OnceCell::new();
 static EUDEX_COMPARAND_HASH: OnceCell<eudex::Hash> = OnceCell::new();
 static REGEX_REPLACE: OnceCell<Regex> = OnceCell::new();
+static SENTIMENT_ANALYZER: OnceCell<SentimentIntensityAnalyzer> = OnceCell::new();
 
 pub fn replace_column_value(
     record: &csv::StringRecord,
@@ -300,12 +308,19 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     }
 
     // validate specified operations
+    let mut censor_invokes = 0_usize;
+    let mut replace_invokes = 0_usize;
+    let mut regex_replace_invokes = 0_usize;
+    let mut sim_invokes = 0_usize;
+    let mut eudex_invokes = 0_usize;
+    let mut sentiment_invokes = 0_usize;
     let operations: Vec<&str> = args.arg_operations.split(',').collect();
     if args.cmd_operations {
         for op in &operations {
             if !OPERATIONS.contains(op) {
                 return fail!(format!("Unknown \"{}\" operation", op));
             }
+            #[allow(clippy::useless_asref)]
             match op.as_ref() {
                 "replace" => {
                     if args.flag_comparand.is_empty() || args.flag_replacement.is_empty() {
@@ -313,6 +328,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                             "--comparand (-C) and --replacement (-R) are required for replace operation."
                         );
                     }
+                    replace_invokes += 1;
                 }
                 "regex_replace" => {
                     if args.flag_comparand.is_empty() || args.flag_replacement.is_empty() {
@@ -320,6 +336,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                             "--comparand (-C) and --replacement (-R) are required for regex_replace operation."
                         );
                     }
+                    regex_replace_invokes += 1;
                 }
                 "copy" => {
                     if args.flag_new_column.is_none() {
@@ -330,6 +347,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                     if args.flag_new_column.is_none() {
                         return fail!("--new_column (-c) is required for censor operations.");
                     }
+                    censor_invokes += 1;
                 }
                 "mtrim" | "mltrim" | "mrtrim" => {
                     if args.flag_comparand.is_empty() {
@@ -340,27 +358,56 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                     if args.flag_new_column.is_none() {
                         return fail!("--new_column (-c) is required for similarity operations.");
                     }
+                    sim_invokes += 1;
                 }
                 "eudex" => {
                     if args.flag_new_column.is_none() {
                         return fail!("--new_column (-c) is required for eudex.");
                     }
+                    eudex_invokes += 1;
+                }
+                "sentiment" => {
+                    if args.flag_new_column.is_none() {
+                        return fail!("--new_column (-c) is required for sentiment operation.");
+                    }
+                    sentiment_invokes += 1;
+                }
+                "whatlang" => {
+                    if args.flag_new_column.is_none() {
+                        return fail!(
+                            "--new_column (-c) is required for whatlang language detection."
+                        );
+                    }
                 }
                 _ => {}
             }
         }
     }
+    if censor_invokes > 1
+        || replace_invokes > 1
+        || regex_replace_invokes > 1
+        || sim_invokes > 1
+        || eudex_invokes > 1
+        || sentiment_invokes > 1
+    {
+        return fail!("you can only use censor, replace, regex_replace, similarity, eudex or sentiment ONCE per operation series.");
+    };
 
     // prep progress bar
-    let mut record_count: u64 = 0;
-    let progress = ProgressBar::new(record_count);
+    let progress = ProgressBar::new(0);
     if !args.flag_quiet {
-        record_count = util::count_rows(&rconfig);
+        let record_count = util::count_rows(&rconfig);
         util::prep_progress(&progress, record_count);
+    } else {
+        progress.set_draw_target(ProgressDrawTarget::hidden());
     }
 
+    #[allow(unused_assignments)]
     let mut record = csv::StringRecord::new();
-    while rdr.read_record(&mut record)? {
+    let records_iter = rdr.records();
+    for result in records_iter {
+        record = result?;
+
         if !args.flag_quiet {
             progress.inc(1);
         }
@@ -418,7 +465,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             let hit_ratio = (hits as f64 / (hits + misses) as f64) * 100.0;
             progress.set_message(format!(
                 " of {} records. Geocode cache hit ratio: {:.2}% - {} entries",
-                record_count.separate_with_commas(),
+                progress.length().separate_with_commas(),
                 hit_ratio,
                 cache_size.separate_with_commas(),
             ));
@@ -431,6 +478,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 #[inline]
 fn apply_operations(operations: &[&str], cell: &mut String, comparand: &str, replacement: &str) {
     for op in operations {
+        #[allow(clippy::useless_asref)]
         match op.as_ref() {
             "len" => {
                 *cell = cell.len().to_string();
@@ -472,14 +520,14 @@ fn apply_operations(operations: &[&str], cell: &mut String, comparand: &str, rep
             }
             "regex_replace" => {
                 let regexreplace =
-                    REGEX_REPLACE.get_or_init(|| regex::Regex::new(&comparand).unwrap());
+                    REGEX_REPLACE.get_or_init(|| regex::Regex::new(comparand).unwrap());
                 *cell = regexreplace.replace_all(cell, replacement).to_string();
             }
             "censor_check" | "censor" => {
                 let censor = CENSOR.get_or_init(|| {
                     let mut censored_words = Censor::Standard + Zealous + Sex;
                     for (_pos, word) in comparand.split(',').enumerate() {
-                        censored_words = censored_words + word.trim();
+                        censored_words += word.trim();
                     }
                     censored_words
                 });
@@ -527,10 +575,28 @@ fn apply_operations(operations: &[&str], cell: &mut String, comparand: &str, rep
             "simod" => *cell = osa_distance(cell, comparand).to_string(),
             "eudex" => {
                 let eudex_comparand_hash =
-                    EUDEX_COMPARAND_HASH.get_or_init(|| eudex::Hash::new(&comparand));
+                    EUDEX_COMPARAND_HASH.get_or_init(|| eudex::Hash::new(comparand));
                 let cell_hash = Hash::new(cell);
                 *cell = format!("{}", (cell_hash - *eudex_comparand_hash).similar());
             }
+            "sentiment" => {
+                let sentiment_analyzer = SENTIMENT_ANALYZER
+                    .get_or_init(vader_sentiment::SentimentIntensityAnalyzer::new);
+                let sentiment_scores = sentiment_analyzer.polarity_scores(cell);
+                *cell = sentiment_scores.get("compound").unwrap().to_string();
+            }
+            "whatlang" => {
+                let lang_info = detect(cell);
+                if let Some(lang_info) = lang_info {
+                    if lang_info.is_reliable() && lang_info.confidence() >= 0.5 {
+                        *cell = format!("{:?}", lang_info.lang());
+                    } else {
+                        // if confidence < 0.5 and !is_reliable(),
+                        // do best-guessed language and add a question mark
+                        *cell = format!("{:?}?", lang_info.lang());
+                    }
+                }
+            }
             _ => {} // this also handles copy, which is a noop
         }
     }
@@ -539,11 +605,12 @@ fn apply_operations(operations: &[&str], cell: &mut String, comparand: &str, rep
 #[cached(
     key = "String",
     convert = r#"{ format!("{}", cell) }"#,
-    option = true
+    option = true,
+    sync_writes = false
 )]
 fn search_cached(cell: &str, formatstr: &str) -> Option<String> {
-    let geocoder = GEOCODER
-        .get_or_init(|| ReverseGeocoder::new(LOCS.get_or_init(|| Locations::from_memory())));
+    let geocoder =
+        GEOCODER.get_or_init(|| ReverseGeocoder::new(LOCS.get_or_init(Locations::from_memory)));
 
     let locregex: &'static Regex =
         regex!(r"(?-u)([+-]?[0-9]+\.?[0-9]*|\.[0-9]+),\s*([+-]?[0-9]+\.?[0-9]*|\.[0-9]+)");
