@@ -1,13 +1,12 @@
 extern crate crossbeam_channel as channel;
-#[cfg(feature = "selfupdate")]
-#[macro_use]
-extern crate self_update;
+use self_update::cargo_crate_version;
 
 use std::borrow::ToOwned;
 use std::env;
 use std::fmt;
 use std::io;
 use std::process;
+use std::time::Instant;
 
 use serde::Deserialize;
 
@@ -48,6 +47,7 @@ macro_rules! command_list {
     enum        Add a new column enumerating CSV lines
     exclude     Excludes the records in one CSV from another
     explode     Explode rows based on some column separator
+    fetch       Create a new column or fetch values from a URL column
     fill        Fill empty values
     fixlengths  Makes all records have same length
     flatten     Show one field per line
@@ -95,12 +95,13 @@ Usage:
     qsv [options]
 
 Options:
-    --list        List all commands available.
-    --envlist     List all environment variables with the QSV_ prefix.
-    --update      Update qsv to the latest release (requires selfupdate feature).
-    -h, --help    Display this message
-    <command> -h  Display the command help message
-    --version     Print version info, mem allocator, max_jobs, num_cpus then exit
+    --list               List all commands available.
+    --envlist            List all environment variables with the QSV_ prefix.
+    -u, --update         Update qsv to the latest release from GitHub.
+    --skip-update-check  Skip automatic update check.
+    -h, --help           Display this message
+    <command> -h         Display the command help message
+    -v, --version        Print version info, mem allocator, max_jobs, num_cpus then exit
 
 Commands:",
     command_list!()
@@ -111,41 +112,84 @@ struct Args {
     arg_command: Option<Command>,
     flag_list: bool,
     flag_envlist: bool,
-    #[allow(dead_code)]
     flag_update: bool,
+    flag_skip_update_check: bool,
 }
 
-#[cfg(feature = "selfupdate")]
-fn get_exec_name() -> Option<String> {
-    std::env::current_exe()
-        .ok()
-        .and_then(|pb| pb.file_name().map(|s| s.to_os_string()))
-        .and_then(|s| s.into_string().ok())
-}
+fn qsv_update(verbose: bool) -> Result<(), Box<dyn ::std::error::Error>> {
+    if env::var("QSV_NO_UPDATE").is_ok() {
+        return Ok(());
+    }
 
-#[cfg(feature = "selfupdate")]
-fn qsv_update() -> Result<(), Box<dyn ::std::error::Error>> {
-    let exec_name = get_exec_name().unwrap();
-    let status = self_update::backends::github::Update::configure()
+    let curr_version = cargo_crate_version!();
+    let releases = self_update::backends::github::ReleaseList::configure()
         .repo_owner("jqnatividad")
         .repo_name("qsv")
-        .bin_name(&exec_name)
-        .show_download_progress(true)
-        .show_output(true)
-        .no_confirm(false)
-        .current_version(cargo_crate_version!())
         .build()?
-        .update()?;
-    let exe_full_path = format!("{:?}", std::env::current_exe().unwrap());
-    println!(
-        "Update status for {}: `{}`!",
-        exe_full_path,
-        status.version()
-    );
+        .fetch()?;
+    let latest_release = &releases[0].version;
+    if latest_release > &curr_version.to_string() {
+        println!(
+            "Update {} available. Current version is {}.",
+            latest_release, curr_version
+        );
+        let status = self_update::backends::github::Update::configure()
+            .repo_owner("jqnatividad")
+            .repo_name("qsv")
+            .bin_name("qsv")
+            .show_download_progress(true)
+            .show_output(verbose)
+            .no_confirm(false)
+            .current_version(curr_version)
+            .build()?
+            .update()?;
+        let exe_full_path = format!("{:?}", std::env::current_exe().unwrap());
+        if verbose {
+            println!(
+                "Update successful for {}: `{}`!",
+                exe_full_path,
+                status.version()
+            );
+        }
+    }
     Ok(())
 }
 
+use flexi_logger::{Cleanup, Criterion, FileSpec, Logger, Naming};
+use log::{error, info, log_enabled, Level};
+
+fn init_logger() {
+    let qsv_log_env = env::var("QSV_LOG_LEVEL").unwrap_or_else(|_| "off".to_string());
+    let qsv_log_dir = env::var("QSV_LOG_DIR").unwrap_or_else(|_| ".".to_string());
+
+    Logger::try_with_env_or_str(qsv_log_env)
+        .unwrap()
+        .log_to_file(
+            FileSpec::default()
+                .directory(qsv_log_dir)
+                .suppress_timestamp(),
+        )
+        .format_for_files(flexi_logger::detailed_format)
+        .o_append(true)
+        .rotate(
+            Criterion::Size(1_000_000),
+            Naming::Numbers,
+            Cleanup::KeepLogAndCompressedFiles(10, 100),
+        )
+        .start()
+        .unwrap();
+}
+
 fn main() {
+    init_logger();
+
+    let now = Instant::now();
+
+    if log_enabled!(Level::Info) {
+        let qsv_args: String = env::args().skip(1).collect::<Vec<_>>().join(" ");
+        info!("START: {}", qsv_args);
+    }
+
     let args: Args = Docopt::new(USAGE)
         .and_then(|d| {
             d.options_first(true)
@@ -160,9 +204,8 @@ fn main() {
         util::show_env_vars();
         return;
     }
-    #[cfg(feature = "selfupdate")]
     if args.flag_update {
-        if let Err(err) = qsv_update() {
+        if let Err(err) = qsv_update(true) {
             werr!("{}", err);
             ::std::process::exit(1);
         }
@@ -176,24 +219,50 @@ fn main() {
 Please choose one of the following commands:",
                 command_list!()
             ));
-            process::exit(0);
+            if !args.flag_skip_update_check {
+                if let Err(err) = qsv_update(false) {
+                    werr!("{}", err);
+                    ::std::process::exit(1);
+                } else {
+                    ::std::process::exit(0);
+                }
+            } else {
+                ::std::process::exit(0);
+            }
         }
         Some(cmd) => match cmd.run() {
-            Ok(()) => process::exit(0),
+            Ok(()) => {
+                if log_enabled!(Level::Info) {
+                    info!("END elapsed: {}", now.elapsed().as_secs_f32());
+                }
+                process::exit(0);
+            }
             Err(CliError::Flag(err)) => err.exit(),
             Err(CliError::Csv(err)) => {
-                werr!("{}", err);
+                if log_enabled!(Level::Error) {
+                    error!("{}", err);
+                } else {
+                    werr!("{}", err);
+                }
                 process::exit(1);
             }
             Err(CliError::Io(ref err)) if err.kind() == io::ErrorKind::BrokenPipe => {
                 process::exit(0);
             }
             Err(CliError::Io(err)) => {
-                werr!("{}", err);
+                if log_enabled!(Level::Error) {
+                    error!("{}", err);
+                } else {
+                    werr!("{}", err);
+                }
                 process::exit(1);
             }
             Err(CliError::Other(msg)) => {
-                werr!("{}", msg);
+                if log_enabled!(Level::Error) {
+                    error!("{}", msg);
+                } else {
+                    werr!("{}", msg);
+                }
                 process::exit(1);
             }
         },
@@ -212,6 +281,7 @@ enum Command {
     Enum,
     Exclude,
     Explode,
+    Fetch,
     Fill,
     FixLengths,
     Flatten,
@@ -268,6 +338,7 @@ impl Command {
             Command::Enum => cmd::enumerate::run(argv),
             Command::Exclude => cmd::exclude::run(argv),
             Command::Explode => cmd::explode::run(argv),
+            Command::Fetch => cmd::fetch::run(argv),
             Command::Fill => cmd::fill::run(argv),
             Command::FixLengths => cmd::fixlengths::run(argv),
             Command::Flatten => cmd::flatten::run(argv),
