@@ -30,6 +30,33 @@ fn apply_ops_upper() {
     assert_eq!(got, expected);
 }
 
+#[test]
+fn apply_ops_empty_shortcircuit() {
+    let wrk = Workdir::new("apply");
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["name"],
+            svec!["John"],
+            svec![""],
+            svec![""],
+            svec!["Hopkins"],
+        ],
+    );
+    let mut cmd = wrk.command("apply");
+    cmd.arg("operations").arg("len").arg("name").arg("data.csv");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["name"],
+        svec!["4"],
+        svec!["0"],
+        svec!["0"],
+        svec!["7"],
+    ];
+    assert_eq!(got, expected);
+}
+
 #[test]
 fn apply_ops_titlecase() {
     let wrk = Workdir::new("apply");
@@ -465,7 +492,7 @@ fn apply_new_column() {
 }
 
 #[test]
-fn apply_currencytonum() {
+fn apply_ops_currencytonum() {
     let wrk = Workdir::new("apply");
     wrk.create(
         "data.csv",
@@ -533,7 +560,7 @@ fn apply_currencytonum() {
 }
 
 #[test]
-fn apply_similarity() {
+fn apply_ops_similarity() {
     let wrk = Workdir::new("apply");
     wrk.create(
         "data.csv",
@@ -567,7 +594,7 @@ fn apply_similarity() {
 }
 
 #[test]
-fn apply_similarity_eudex() {
+fn apply_ops_similarity_eudex() {
     let wrk = Workdir::new("apply");
     wrk.create(
         "data.csv",
@@ -603,7 +630,7 @@ fn apply_similarity_eudex() {
 }
 
 #[test]
-fn apply_similarity_more_eudex() {
+fn apply_ops_similarity_more_eudex() {
     let wrk = Workdir::new("apply");
     wrk.create(
         "data.csv",
@@ -640,6 +667,81 @@ fn apply_similarity_more_eudex() {
     assert_eq!(got, expected);
 }
 
+#[test]
+fn apply_ops_sentiment() {
+    let wrk = Workdir::new("apply");
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["customer comment"],
+            svec!["This is ridiculous! I will never buy from this company again!"],
+            svec!["Josephina was awesome! She was very helpful and patient. I wish more customer service folks are like her!"],
+            svec!["I can't believe that garbage is still out there. That is so false!"],
+            svec!["5 stars! Highly recommended!"],
+            svec!["What were they thinking!?!"],
+        ],
+    );
+    let mut cmd = wrk.command("apply");
+    cmd.arg("operations")
+        .arg("sentiment")
+        .arg("customer comment")
+        .arg("--new-column")
+        .arg("sentiment_score")
+        .arg("data.csv");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["customer comment", "sentiment_score"],
+        svec!["This is ridiculous! I will never buy from this company again!", "-0.47384376462380107"],
+        svec!["Josephina was awesome! She was very helpful and patient. I wish more customer service folks are like her!", "0.9227060290926788"],
+        svec!["I can't believe that garbage is still out there. That is so false!", "-0.07518070500292766"],
+        svec!["5 stars! Highly recommended!", "0.3973495344831422"],
+        svec!["What were they thinking!?!", "-0.19353437967075598"],
+    ];
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn apply_ops_whatlang() {
+    let wrk = Workdir::new("apply");
+    wrk.create(
+        "data.csv",
+       vec![
+            svec!["description"],
+            svec!["Y así mismo, aunque no son tan ágiles en el suelo como el vampiro común, son muy competentes al escalar por las ramas."],
+            svec!["See notes."],
+            svec!["Aquest és l’honor més gran que he rebut a la meva vida. La pau ha estat sempre la meva més gran preocupació."],
+            svec![""],
+            svec!["Showing that even in the modern warfare of the 1930s and 1940s, the dilapidated fortifications still had defensive usefulness."],
+            svec!["民國卅八年（ 1949年 ）， 從南京經 廣州 、 香港返回 香日德。 1950年6月 ，受十世班禪派遣， 前往西安代表班禪向彭德懷投誠 。"],
+            svec!["Rust（ラスト）は並列かつマルチパラダイムのプログラミング言語である"],
+            svec!["Мой дядя самых честных правил, Когда не в шутку занемог, Он уважать себя заставил И лучше выдумать не мог."],
+        ],
+    );
+    let mut cmd = wrk.command("apply");
+    cmd.arg("operations")
+        .arg("whatlang")
+        .arg("description")
+        .arg("--new-column")
+        .arg("language")
+        .arg("data.csv");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["description", "language"],
+        svec!["Y así mismo, aunque no son tan ágiles en el suelo como el vampiro común, son muy competentes al escalar por las ramas.", "Spa"],
+        svec!["See notes.", "Cat?"],
+        svec!["Aquest és l’honor més gran que he rebut a la meva vida. La pau ha estat sempre la meva més gran preocupació.", "Cat"],
+        svec!["", ""],
+        svec!["Showing that even in the modern warfare of the 1930s and 1940s, the dilapidated fortifications still had defensive usefulness.", "Eng"],
+        svec!["民國卅八年（ 1949年 ）， 從南京經 廣州 、 香港返回 香日德。 1950年6月 ，受十世班禪派遣， 前往西安代表班禪向彭德懷投誠 。", "Cmn"],
+        svec!["Rust（ラスト）は並列かつマルチパラダイムのプログラミング言語である", "Jpn"],
+        svec!["Мой дядя самых честных правил, Когда не в шутку занемог, Он уважать себя заставил И лучше выдумать не мог.", "Rus"],
+
+  ];
+    assert_eq!(got, expected);
+}
+
 #[test]
 fn apply_emptyreplace() {
     let wrk = Workdir::new("apply");
