@@ -71,7 +71,7 @@ fn test_stats<S>(
     let field_val = get_field_value(&wrk, &mut cmd, field);
     // Only compare the first few bytes since floating point arithmetic
     // can mess with exact comparisons.
-    let len = cmp::min(10, cmp::min(field_val.len(), expected.len()));
+    let len = cmp::min(11, cmp::min(field_val.len(), expected.len()));
     assert_eq!(&field_val[0..len], &expected[0..len]);
 }
 
@@ -115,7 +115,9 @@ fn get_field_value(wrk: &Workdir, cmd: &mut process::Command, field: &str) -> St
     if field == "median" {
         cmd.arg("--median");
     }
-    if field == "quartiles" { cmd.arg("--quartiles"); }
+    if field == "quartiles" {
+        cmd.arg("--quartiles");
+    }
     if field == "cardinality" {
         cmd.arg("--cardinality");
     }
@@ -129,21 +131,21 @@ fn get_field_value(wrk: &Workdir, cmd: &mut process::Command, field: &str) -> St
     for row in rows.iter() {
         for (h, val) in headers.iter().zip(row.iter()) {
             match field {
-                "quartiles" => {
-                    match &**h {
-                        "q1" | "q2" => {
-                            sequence.push(val);
-                        },
-                        "q3" => {
-                            sequence.push(val);
-                            return sequence.join(",").clone();
-                        },
-                        _ => {},
+                "quartiles" => match &**h {
+                    "lower_fence" | "q1" | "q2_median" | "q3" | "iqr" => {
+                        sequence.push(val);
                     }
+                    "upper_fence" => {
+                        sequence.push(val);
+                        return sequence.join(",").clone();
+                    }
+                    _ => {}
                 },
                 _ => {
-                    if &**h == field { return val.clone(); }
-                },
+                    if &**h == field {
+                        return val.clone();
+                    }
+                }
             }
         }
     }
@@ -204,14 +206,26 @@ stats_tests!(stats_no_mean, "mean", &["a"], "");
 stats_tests!(stats_no_stddev, "stddev", &["a"], "");
 stats_tests!(stats_no_variance, "variance", &["a"], "");
 stats_tests!(stats_no_median, "median", &["a"], "");
-stats_tests!(stats_no_quartiles, "quartiles", &["a"], ",,");
+stats_tests!(stats_no_quartiles, "quartiles", &["a"], ",,,,,");
 stats_tests!(stats_no_mode, "mode", &["a", "b"], "N/A");
+stats_tests!(
+    stats_multiple_modes,
+    "mode",
+    &["a", "a", "b", "b", "c", "d", "e", "e"],
+    "a,b,e"
+);
+stats_tests!(
+    stats_multiple_modes_num,
+    "mode",
+    &["5", "5", "33", "33", "42", "17", "99", "99"],
+    "33,5,99"
+);
 
 stats_tests!(stats_null_mean, "mean", &[""], "");
 stats_tests!(stats_null_stddev, "stddev", &[""], "");
 stats_tests!(stats_null_variance, "variance", &[""], "");
 stats_tests!(stats_null_median, "median", &[""], "");
-stats_tests!(stats_null_quartiles, "quartiles", &[""], ",,");
+stats_tests!(stats_null_quartiles, "quartiles", &[""], ",,,,,");
 stats_tests!(stats_null_mode, "mode", &[""], "N/A");
 
 stats_tests!(stats_includenulls_null_mean, "mean", &[""], "", true);
@@ -224,7 +238,13 @@ stats_tests!(
     true
 );
 stats_tests!(stats_includenulls_null_median, "median", &[""], "", true);
-stats_tests!(stats_includenulls_null_quartiles, "quartiles", &[""], ",,", true);
+stats_tests!(
+    stats_includenulls_null_quartiles,
+    "quartiles",
+    &[""],
+    ",,,,,",
+    true
+);
 stats_tests!(stats_includenulls_null_mode, "mode", &[""], "N/A", true);
 
 stats_tests!(
@@ -303,11 +323,36 @@ stats_tests!(
     "2.5"
 );
 stats_tests!(stats_median_mix, "median", &["1", "2.5", "3"], "2.5");
-stats_tests!(stats_quartiles, "quartiles", &["1", "2", "3"], "1,2,3");
-stats_tests!(stats_quartiles_null, "quartiles", &["", "1", "2", "3"], "1,2,3");
-stats_tests!(stats_quartiles_even, "quartiles", &["1", "2", "3", "4"], "1.5,2.5,3.5");
-stats_tests!(stats_quartiles_even_null, "quartiles", &["", "1", "2", "3", "4"], "1.5,2.5,3.5");
-stats_tests!(stats_quartiles_mix, "quartiles", &["1", "2.0", "3", "4"], "1.5,2.5,3.5");
+stats_tests!(
+    stats_quartiles,
+    "quartiles",
+    &["1", "2", "3"],
+    "-2,1,2,3,2,4"
+);
+stats_tests!(
+    stats_quartiles_null,
+    "quartiles",
+    &["", "1", "2", "3"],
+    "-2,1,2,3,2,4"
+);
+stats_tests!(
+    stats_quartiles_even,
+    "quartiles",
+    &["1", "2", "3", "4"],
+    "-1.5,1.5,2.5,3.5,1,4.5"
+);
+stats_tests!(
+    stats_quartiles_even_null,
+    "quartiles",
+    &["", "1", "2", "3", "4"],
+    "-1.5,1.5,2.5,3.5,1,4.5"
+);
+stats_tests!(
+    stats_quartiles_mix,
+    "quartiles",
+    &["1", "2.0", "3", "4"],
+    "-1.5,1.5,2.5,3.5,1,4.5"
+);
 
 stats_tests!(stats_nullcount, "nullcount", &["", "1", "2"], "1");
 stats_tests!(stats_nullcount_none, "nullcount", &["a", "1", "2"], "0");
@@ -348,7 +393,7 @@ mod stats_zero_median {
 
 mod stats_zero_quartiles {
     use super::test_stats;
-    stats_test_headers!(stats_zero_quartiles, "quartiles", &[], ",,");
+    stats_test_headers!(stats_zero_quartiles, "quartiles", &[], ",,,,,");
 }
 
 mod stats_header_fields {
